@@ -25,7 +25,7 @@
 ######################################################################
 
 fitabn <- function(data.df,dag.m,hyper.params=list("mean"=c(0),"sd"=c(sqrt(1000)),"shape"=c(0.001),"scale"=c(1/0.001)),max.iters=100,epsabs=1e-7, 
-                   verbose=FALSE, error.verbose=FALSE) {
+                   verbose=FALSE, error.verbose=FALSE,std=TRUE) {
     
     #need to find categorical variables and continuous - Gaussian
     get.factors<-NULL; 
@@ -41,7 +41,7 @@ fitabn <- function(data.df,dag.m,hyper.params=list("mean"=c(0),"sd"=c(sqrt(1000)
     var.types<-as.integer(var.types);#1-cat, 0-numeric
     
     if(length(get.factors)>0){#have some categorical variables so some coersion to factors then integers needed
-    data.df.cat<-data.df[,get.factors];
+    data.df.cat<-as.data.frame(data.df[,get.factors]);names(data.df.cat)<-names(data.df)[get.factors];### NOTE as.data.frame() is NEW
     obsdata.cat<-makeintofactors(data.df.cat);## need to make into integers
     numVarLevels<-as.integer(apply(obsdata.cat,2,max));## get vector of number of levels in each variables
     if( length(which(numVarLevels!=2)) ){#cat("Error in ",names(data.df)[which(min(numVarLevels)==1)],"\n");
@@ -49,8 +49,10 @@ fitabn <- function(data.df,dag.m,hyper.params=list("mean"=c(0),"sd"=c(sqrt(1000)
     }   
     
     if(length(get.gaussian)>0){
-                               obsdata.cts<-data.df[,get.gaussian];
-                               for(i in 1:dim(obsdata.cts)[2]){obsdata.cts[,i]<-as.double(obsdata.cts[,i]);}
+                               obsdata.cts<-as.data.frame(data.df[,get.gaussian]);names(obsdata.cts)<-names(data.df)[get.gaussian];### NOTE as.data.frame() is NEW
+                               for(i in 1:dim(obsdata.cts)[2]){if(std){# std. gaus vars to mean zero and sd=1
+                                                               obsdata.cts[,i]<- (obsdata.cts[,i]-mean(obsdata.cts[,i]))/sd(obsdata.cts[,i]);}
+                                                               obsdata.cts[,i]<-as.double(obsdata.cts[,i]);}
     }
     
     #now put all the data back into a single data frame
@@ -71,23 +73,23 @@ fitabn <- function(data.df,dag.m,hyper.params=list("mean"=c(0),"sd"=c(sqrt(1000)
     if(verbose){loc.verbose<-1;} else {loc.verbose<-0;}
     loc.verbose<-as.integer(loc.verbose);
 
-    if(!is.logical(error.verbose)){stop("erorr.verbose must be either TRUE or FALSE");}
+    if(!is.logical(error.verbose)){stop("error.verbose must be either TRUE or FALSE");}
     if(error.verbose){loc.error.verbose<-1;} else {loc.error.verbose<-0;}
     loc.error.verbose<-as.integer(loc.error.verbose);
 
     ## coerce list into two vectors, one for means and one for **standard deviation** 
     prior.mean<-hyper.params$mean;
     if(length(prior.mean)!=dim(data.df)[2]+1){prior.mean<-as.double(rep(0.0,dim(data.df)[2]+1));}      
-    if(length(prior.mean)!=dim(data.df)[2]+1){stop("need prior mean for each variable in data.frame");}
+    #if(length(prior.mean)!=dim(data.df)[2]+1){stop("need prior mean for each variable in data.frame");}
     
     prior.sd  <-hyper.params$sd;
     if(length(prior.sd)!=dim(data.df)[2]+1){prior.sd<-as.double(rep(sqrt(1000.0),dim(data.df)[2]+1));}      
-    if(length(prior.sd)!=dim(data.df)[2]+1){stop("need prior mean for each variable in data.frame");}
+    #if(length(prior.sd)!=dim(data.df)[2]+1){stop("need prior mean for each variable in data.frame");}
     
-    if(is.null(hyper.params$shape)){hyper.params$shape=0.001;}
+    #if(is.null(hyper.params$shape)){hyper.params$shape=0.001;}
     prior.gamma.shape<-hyper.params$shape;
     if(length(get.gaussian)>0 && length(prior.gamma.shape)!=length(get.gaussian)){prior.gamma.shape<-as.double(rep(0.001,length(get.gaussian)));}
-    if(is.null(hyper.params$scale)){hyper.params$scale=1/0.001;}  
+    #if(is.null(hyper.params$scale)){hyper.params$scale=1/0.001;}  
     prior.gamma.scale<-hyper.params$scale;
     if(length(get.gaussian)>0 && length(prior.gamma.scale)!=length(get.gaussian)){prior.gamma.scale<-as.double(rep(1/0.001,length(get.gaussian)));}
     
